@@ -1067,14 +1067,16 @@ sr_lydmods_sched_update_data(const struct lyd_node *sr_mods, const struct ly_ctx
     sr_error_info_t *err_info = NULL;
     struct lyd_node *old_start_data = NULL, *new_start_data = NULL, *old_run_data = NULL, *new_run_data = NULL, *mod_data;
     struct ly_ctx *old_ctx = NULL;
-    struct ly_set *set = NULL, *startup_set = NULL;
-    const struct lys_module *ly_mod;
+    struct ly_set *mod_set = NULL, *del_mod_set = NULL, *startup_set = NULL;
+    const struct lys_module *ly_mod, *ly_mod2;
     char *start_data_json = NULL, *run_data_json = NULL, *path;
     uint32_t idx;
     int exists;
 
-    set = ly_set_new();
-    SR_CHECK_MEM_GOTO(!set, err_info, cleanup);
+    mod_set = ly_set_new();
+    SR_CHECK_MEM_GOTO(!mod_set, err_info, cleanup);
+    del_mod_set = ly_set_new();
+    SR_CHECK_MEM_GOTO(!del_mod_set, err_info, cleanup);
 
     /* first build context without any scheduled changes */
     if ((err_info = sr_ly_ctx_new(&old_ctx))) {
@@ -1111,12 +1113,14 @@ sr_lydmods_sched_update_data(const struct lyd_node *sr_mods, const struct ly_ctx
             }
         }
 
-        /* remember this module from the new context */
-        ly_mod = ly_ctx_get_module(new_ctx, ly_mod->name, NULL, 1);
-        if (ly_mod) {
-            assert(ly_mod->implemented);
-            ly_set_add(set, (void *)ly_mod, LY_SET_OPT_USEASLIST);
-        } /* else the module was removed */
+        ly_mod2 = ly_ctx_get_module(new_ctx, ly_mod->name, NULL, 1);
+        if (ly_mod2) {
+            /* remember this module from the new context */
+            ly_set_add(mod_set, (void *)ly_mod2, LY_SET_OPT_USEASLIST);
+        } else {
+            /* module was removed, remember it as well */
+            ly_set_add(del_mod_set, (void *)ly_mod, LY_SET_OPT_USEASLIST);
+        }
     }
 
     /* print the data of all the modules into JSON */
@@ -1161,7 +1165,7 @@ sr_lydmods_sched_update_data(const struct lyd_node *sr_mods, const struct ly_ctx
         }
 
         /* remember this module */
-        ly_set_add(set, (void *)lyd_node_module(mod_data), LY_SET_OPT_USEASLIST);
+        ly_set_add(mod_set, (void *)lyd_node_module(mod_data), LY_SET_OPT_USEASLIST);
 
         /* link to the new startup/running data */
         if (!new_start_data) {
@@ -1188,8 +1192,8 @@ sr_lydmods_sched_update_data(const struct lyd_node *sr_mods, const struct ly_ctx
     }
 
     /* print all modules data with the updated module context and free them, no longer needed */
-    for (idx = 0; idx < set->number; ++idx) {
-        ly_mod = (struct lys_module *)set->set.g[idx];
+    for (idx = 0; idx < mod_set->number; ++idx) {
+        ly_mod = (struct lys_module *)mod_set->set.g[idx];
 
         /* startup data */
         mod_data = sr_module_data_unlink(&new_start_data, ly_mod);
@@ -1211,7 +1215,8 @@ sr_lydmods_sched_update_data(const struct lyd_node *sr_mods, const struct ly_ctx
     /* success */
 
 cleanup:
-    ly_set_free(set);
+    ly_set_free(mod_set);
+    ly_set_free(del_mod_set);
     ly_set_free(startup_set);
     lyd_free_withsiblings(old_start_data);
     lyd_free_withsiblings(new_start_data);
@@ -1526,10 +1531,7 @@ sr_lydmods_add_all(struct lyd_node *sr_mod, const struct lys_module *ly_mod)
         /* find the dependent module */
         set2 = lyd_find_path(sr_mod->parent, xpath);
         free(xpath);
-        if (!set2) {
-            SR_ERRINFO_MEM(&err_info);
-            goto cleanup;
-        }
+        SR_CHECK_LY_GOTO(!set2, ly_mod->ctx, err_info, cleanup);
         assert(set2->number == 1);
 
         /* add inverse dependency */
