@@ -131,7 +131,7 @@ sr_replay_open_file(const char *mod_name, time_t from_ts, time_t to_ts, int flag
 {
     sr_error_info_t *err_info = NULL;
     char *path = NULL;
-    mode_t perm = SR_FILE_PERM, um;
+    mode_t perm = SR_FILE_PERM;
 
     *notif_fd = -1;
 
@@ -146,13 +146,9 @@ sr_replay_open_file(const char *mod_name, time_t from_ts, time_t to_ts, int flag
         goto cleanup;
     }
 
-    /* set umask so that the correct permissions are really set */
-    um = umask(SR_UMASK);
-
-    *notif_fd = SR_OPEN(path, flags, perm);
-    umask(um);
+    *notif_fd = sr_open(path, flags, perm);
     if (*notif_fd == -1) {
-        sr_errinfo_new(&err_info, SR_ERR_SYS, NULL, "Failed to open file \"%s\" (%s).", path, strerror(errno));
+        SR_ERRINFO_OPEN(&err_info, path);
         goto cleanup;
     }
 
@@ -710,8 +706,8 @@ sr_replay_notify(sr_conn_ctx_t *conn, const char *mod_name, const char *xpath, t
     time_t file_from_ts, file_to_ts, notif_ts;
     struct ly_set *set = NULL;
     struct lyd_node *notif = NULL, *notif_op;
+    sr_session_ctx_t *ev_sess = NULL;
     int fd = -1;
-    sr_sid_t sid = {0};
 
     /* find SHM mod for replay lock and check if replay is even supported */
     shm_mod = sr_shmmain_find_module(SR_CONN_MAIN_SHM(conn), mod_name);
@@ -723,6 +719,11 @@ sr_replay_notify(sr_conn_ctx_t *conn, const char *mod_name, const char *xpath, t
         goto cleanup;
     }
 
+    /* create event session */
+    if ((err_info = _sr_session_start(conn, SR_DS_OPERATIONAL, SR_SUB_EV_NOTIF, 0, 0, NULL, &ev_sess))) {
+        goto cleanup;
+    }
+
     /* find first file */
     if ((err_info = sr_replay_find_file(mod_name, start_time, 0, &file_from_ts, &file_to_ts))) {
         goto cleanup;
@@ -774,8 +775,8 @@ sr_replay_notify(sr_conn_ctx_t *conn, const char *mod_name, const char *xpath, t
                 SR_CHECK_INT_GOTO(notif_op->schema->nodetype != LYS_NOTIF, err_info, cleanup);
 
                 /* call callback */
-                if ((err_info = sr_notif_call_callback(conn, cb, tree_cb, private_data, SR_EV_NOTIF_REPLAY, notif_op,
-                        notif_ts, sid))) {
+                if ((err_info = sr_notif_call_callback(ev_sess, cb, tree_cb, private_data, SR_EV_NOTIF_REPLAY, notif_op,
+                        notif_ts))) {
                     goto cleanup;
                 }
             }
@@ -799,8 +800,8 @@ sr_replay_notify(sr_conn_ctx_t *conn, const char *mod_name, const char *xpath, t
 
     /* replay last notification if the subscription continues */
     notif_ts = time(NULL);
-    if ((!stop_time || (stop_time >= notif_ts)) && (err_info = sr_notif_call_callback(conn, cb, tree_cb, private_data,
-            SR_EV_NOTIF_REPLAY_COMPLETE, NULL, stop_time ? stop_time : notif_ts, sid))) {
+    if ((!stop_time || (stop_time >= notif_ts)) && (err_info = sr_notif_call_callback(ev_sess, cb, tree_cb, private_data,
+            SR_EV_NOTIF_REPLAY_COMPLETE, NULL, stop_time ? stop_time : notif_ts))) {
         goto cleanup;
     }
 
@@ -810,6 +811,7 @@ cleanup:
     if (fd > -1) {
         close(fd);
     }
+    sr_session_stop(ev_sess);
     lyd_free_withsiblings(notif);
     ly_set_free(set);
     return err_info;
