@@ -32,26 +32,13 @@
 #include "common.h"
 
 #define SR_MAIN_SHM_LOCK "sr_main_lock"     /**< Main SHM file lock name. */
-#define SR_SHM_VER 5                        /**< Main and ext SHM version of their expected content structures. */
+#define SR_SHM_VER 6                        /**< Main and ext SHM version of their expected content structures. */
 
 /**
  * Main SHM organization
  *
  * Except for main and ext SHM there are individual SHM segments for subscriptions and
- * running data files. These are not covered in the following text.
- *
- * There are 2 SHM segments, main SHM and ext SHM.
- *
- * Main SHM starts with ::sr_main_shm_t structure. Then is followed by all installed
- * modules, each with a ::sr_mod_t structure until the end of main SHM. All `off_t`
- * types in these structures are offset pointers to ext SHM.
- *
- * Ext shm starts with a `size_t` value representing the number of wasted
- * bytes in this SHM segment. It is followed by arrays and strings pointed to
- * by main SHM `off_t` pointers. First, there is the sysrepo connections state ::sr_conn_shm_t
- * meaning all currently running connections. Then, there is information from ::sr_mod_t
- * which includes names, dependencies, and subscriptions. Lastly, there are RPCs ::sr_rpc_t.
- * Also, any pointers in all the previous structures point, again, into ext SHM.
+ * running data files.
  */
 
 /**
@@ -155,7 +142,7 @@ typedef struct sr_main_shm_s {
     uint32_t shm_ver;           /**< Main and ext SHM version of all expected data stored in them. Is increased with
                                      every change of their structure content (ABI change). */
     pthread_mutex_t lydmods_lock; /**< Process-shared lock for accessing sysrepo module data. */
-    pthread_mutex_t ext_lock;   /**< Process-shared lock for truncating (allocating more) ext SHM. */
+    pthread_mutex_t ext_lock;   /**< Process-shared lock for accessing holes and truncating ext SHM. */
     uint32_t mod_count;         /**< Number of installed modules stored after this structure. */
 
     ATOMIC_T new_sr_cid;        /**< Connection ID for a new connection. */
@@ -218,12 +205,20 @@ typedef struct sr_mod_rpc_sub_s {
 } sr_mod_rpc_sub_t;
 
 /**
- * @brief External (ext) SHM.
+ * @brief Ext SHM structure.
  */
 typedef struct sr_ext_shm_s {
-    ATOMIC_T wasted;            /**< Number of unused allocated bytes in the memory. */
+    uint32_t first_hole_off;    /**< Offset of the first memory hole, 0 if there is none. */
 } sr_ext_shm_t;
 
+/**
+ * @brief Ext SHM memory hole.
+ */
+typedef struct sr_ext_hole_s {
+    uint32_t size;
+    uint32_t next_hole_off;
+} sr_ext_hole_t;
+
 /**
  * @brief Subscription event.
  */
@@ -258,7 +253,8 @@ typedef struct sr_sub_shm_s {
 
     uint32_t request_id;        /**< Request ID. */
     sr_sub_event_t event;       /**< Event. */
-    sr_sid_t sid;               /**< Originator SID information. */
+    uint32_t sid;               /**< Originator sysrepo session ID. */
+    uint32_t ncid;              /**< Originator NETCONF session ID. */
 } sr_sub_shm_t;
 
 /**
@@ -269,7 +265,8 @@ typedef struct sr_multi_sub_shm_s {
 
     uint32_t request_id;        /**< Request ID. */
     sr_sub_event_t event;       /**< Event. */
-    sr_sid_t sid;               /**< Originator SID information. */
+    uint32_t sid;               /**< Originator sysrepo session ID. */
+    uint32_t ncid;              /**< Originator NETCONF session ID. */
 
     /* specific fields */
     uint32_t priority;          /**< Priority of the subscriber. */
@@ -278,12 +275,12 @@ typedef struct sr_multi_sub_shm_s {
 /*
  * change data subscription SHM (multi)
  *
- * FOR SUBSCRIBERS
- * followed by:
- * event SR_SUB_EV_UPDATE, SR_SUB_EV_CHANGE, SR_SUB_EV_DONE, SR_SUB_EV_ABORT - char *diff_lyb - diff tree
+ * data SHM contents
  *
- * FOR ORIGINATOR (when subscriber_count is 0)
- * followed by:
+ * FOR SUBSCRIBERS:
+ * event SR_SUB_EV_UPDATE, SR_SUB_EV_CHANGE, SR_SUB_EV_DONE, SR_SUB_EV_ABORT - char *user; char *diff_lyb - diff tree
+ *
+ * FOR ORIGINATOR (when subscriber_count is 0):
  * event SR_SUB_EV_SUCCESS - char *edit_lyb
  * event SR_SUB_EV_ERROR - char *error_message; char *error_xpath
  */
@@ -291,17 +288,21 @@ typedef struct sr_multi_sub_shm_s {
 /*
  * notification subscription SHM (multi)
  *
+ * data SHM contents
+ *
  * FOR SUBSCRIBERS
  * followed by:
- * event SR_SUB_EV_NOTIF - time_t notif_timestamp; char *notif_lyb - notification
+ * event SR_SUB_EV_NOTIF - char *user; time_t notif_timestamp; char *notif_lyb - notification
  */
 
 /*
  * operational subscription SHM (generic)
  *
+ * data SHM contents
+ *
  * FOR SUBSCRIBER
  * followed by:
- * event SR_SUB_EV_OPER - char *request_xpath; char *parent_lyb - existing data tree parent
+ * event SR_SUB_EV_OPER - char *user; char *request_xpath; char *parent_lyb - existing data tree parent
  *
  * FOR ORIGINATOR
  * followed by:
@@ -312,9 +313,11 @@ typedef struct sr_multi_sub_shm_s {
 /*
  * RPC subscription SHM (generic)
  *
+ * data SHM contents
+ *
  * FOR SUBSCRIBER
  * followed by:
- * event SR_SUB_EV_RPC - char *input_lyb - RPC/action with input
+ * event SR_SUB_EV_RPC - char *user; char *input_lyb - RPC/action with input
  *
  * FOR ORIGINATOR
  * followed by:
@@ -492,44 +495,53 @@ sr_error_info_t *sr_shmmain_check_data_files(sr_main_shm_t *main_shm);
  *
  * @param[in] conn Connection to use.
  * @param[in] mode Mode of the remap lock.
- * @param[in] ext_lock Whether ext SHM will also be truncated (enlarged, new memory allocated) when ext lock will
- * be locked or not.
+ * @param[in] ext_lock Whether to lock ext lock.
  * @param[in] func Caller function name for logging.
  * @return err_info, NULL on success.
  */
 sr_error_info_t *sr_shmext_conn_remap_lock(sr_conn_ctx_t *conn, sr_lock_mode_t mode, int ext_lock, const char *func);
 
 /**
- * @brief Unlock ext SHM lock and connection remap lock.
+ * @brief Unlock ext SHM lock and connection remap lock, truncate ext SHM if possible.
  *
  * @param[in] conn Connection to use.
- * @param[in] mode Mode of the remap lock.
- * @param[in] ext_lock Whether to unlock ext lock or not.
+ * @param[in] mode Mode of the ext and remap lock.
+ * @param[in] ext_lock Whether to unlock ext lock.
  * @param[in] func Caller function name for logging.
  */
 void sr_shmext_conn_remap_unlock(sr_conn_ctx_t *conn, sr_lock_mode_t mode, int ext_lock, const char *func);
 
+/**
+ * @brief Debug print the contents of ext SHM.
+ *
+ * @param[in] main_shm Main SHM.
+ * @param[in] shm_ext Ext SHM.
+ */
+void sr_shmext_print(sr_main_shm_t *main_shm, sr_shm_t *shm_ext);
+
 /**
  * @brief Add main SHM module change subscription and create sub SHM if the first subscription was added.
  * Ext SHM may be remapped!
  *
  * @param[in] conn Connection to use.
  * @param[in] shm_mod SHM module.
- * @param[in] xpath Subscription XPath.
+ * @param[in] has_lock Whether CHANGE SUB lock is already held.
  * @param[in] ds Datastore.
+ * @param[in] xpath Subscription XPath.
  * @param[in] priority Subscription priority.
  * @param[in] sub_opts Subscription options.
  * @param[in] evpipe_num Subscription event pipe number.
  * @return err_info, NULL on success.
  */
-sr_error_info_t *sr_shmext_change_subscription_add(sr_conn_ctx_t *conn, sr_mod_t *shm_mod, const char *xpath,
-        sr_datastore_t ds, uint32_t priority, int sub_opts, uint32_t evpipe_num);
+sr_error_info_t *sr_shmext_change_subscription_add(sr_conn_ctx_t *conn, sr_mod_t *shm_mod, sr_lock_mode_t has_lock,
+        sr_datastore_t ds, const char *xpath, uint32_t priority, int sub_opts, uint32_t evpipe_num);
 
 /**
  * @brief Remove main SHM module change subscription and unlink sub SHM if the last subscription was removed.
  *
  * @param[in] conn Connection to use.
  * @param[in] shm_mod SHM module.
+ * @param[in] has_lock Whether CHANGE SUB lock is already held.
  * @param[in] ds Datastore.
  * @param[in] xpath Subscription XPath.
  * @param[in] priority Subscription priority.
@@ -537,8 +549,8 @@ sr_error_info_t *sr_shmext_change_subscription_add(sr_conn_ctx_t *conn, sr_mod_t
  * @param[in] evpipe_num Subscription event pipe number.
  * @return err_info, NULL on success.
  */
-sr_error_info_t *sr_shmext_change_subscription_del(sr_conn_ctx_t *conn, sr_mod_t *shm_mod, sr_datastore_t ds,
-        const char *xpath, uint32_t priority, int sub_opts, uint32_t evpipe_num);
+sr_error_info_t *sr_shmext_change_subscription_del(sr_conn_ctx_t *conn, sr_mod_t *shm_mod, sr_lock_mode_t has_lock,
+        sr_datastore_t ds, const char *xpath, uint32_t priority, int sub_opts, uint32_t evpipe_num);
 
 /**
  * @brief Remove main SHM module change subscription with param-based cleanup.
@@ -879,6 +891,16 @@ sr_error_info_t *sr_shmsub_open_map(const char *name, const char *suffix1, int64
  */
 sr_error_info_t *sr_shmsub_unlink(const char *name, const char *suffix1, int64_t suffix2);
 
+/**
+ * @brief Unlink a subscription data SHM.
+ *
+ * @param[in] name Subscription name (module name).
+ * @param[in] suffix1 First suffix.
+ * @param[in] suffix2 Second suffix, none if set to -1.
+ * @return err_info, NULL on success.
+ */
+sr_error_info_t *sr_shmsub_data_unlink(const char *name, const char *suffix1, int64_t suffix2);
+
 /**
  * @brief Write into a subscriber event pipe to notify it there is a new event.
  *
