@@ -208,7 +208,7 @@ test_data_deps(void **state)
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>test</name>"
         "<removed/>"
-        "<inverse-data-deps>refs</inverse-data-deps>"
+        "<inverse-deps>refs</inverse-deps>"
     "</module>"
     );
     cmp_int_data(st->conn, "ietf-interfaces",
@@ -231,16 +231,16 @@ test_data_deps(void **state)
         "<name>refs</name>"
         "<replay-support>0000000000</replay-support>"
         "<removed/>"
-        "<data-deps>"
+        "<deps>"
             "<module>test</module>"
             "<inst-id>"
-                "<xpath xmlns:r=\"urn:refs\">/r:cont/r:def-inst-id</xpath>"
+                "<path xmlns:r=\"urn:refs\">/r:cont/r:def-inst-id</path>"
                 "<default-module>test</default-module>"
             "</inst-id>"
             "<inst-id>"
-                "<xpath xmlns:r=\"urn:refs\">/r:inst-id</xpath>"
+                "<path xmlns:r=\"urn:refs\">/r:inst-id</path>"
             "</inst-id>"
-        "</data-deps>"
+        "</deps>"
     "</module>"
     );
 }
@@ -282,46 +282,46 @@ test_op_deps(void **state)
     cmp_int_data(st->conn, "ops",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>ops</name>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1</xpath>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1</path>"
             "<out>"
                 "<module>ops</module>"
                 "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1/o:l8</xpath>"
+                    "<path xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1/o:l8</path>"
                     "<default-module>ops</default-module>"
                 "</inst-id>"
             "</out>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2</xpath>"
-            "<in>"
-                "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2/o:l13</xpath>"
-                "</inst-id>"
-            "</in>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:rpc1</xpath>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:rpc1</path>"
             "<in>"
                 "<module>ops-ref</module>"
                 "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:rpc1/o:l2</xpath>"
+                    "<path xmlns:o=\"urn:ops\">/o:rpc1/o:l2</path>"
                     "<default-module>ops-ref</default-module>"
                 "</inst-id>"
             "</in>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:rpc2</xpath>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:rpc2</path>"
             "<out>"
                 "<module>ops-ref</module>"
             "</out>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:rpc3</xpath>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:notif4</xpath>"
-        "</op-deps>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:rpc3</path>"
+        "</rpc>"
+        "<notification>"
+            "<path xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2</path>"
+            "<deps>"
+                "<inst-id>"
+                    "<path xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2/o:l13</path>"
+                "</inst-id>"
+            "</deps>"
+        "</notification>"
+        "<notification>"
+            "<path xmlns:o=\"urn:ops\">/o:notif4</path>"
+        "</notification>"
     "</module>"
     );
 
@@ -341,59 +341,59 @@ test_op_deps(void **state)
     cmp_int_data(st->conn, "ops",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>ops</name>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1</xpath>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1</path>"
             "<out>"
                 "<module>ops</module>"
                 "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1/o:l8</xpath>"
+                    "<path xmlns:o=\"urn:ops\">/o:cont/o:list1/o:cont2/o:act1/o:l8</path>"
                     "<default-module>ops</default-module>"
                 "</inst-id>"
             "</out>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:cont/o:list1/o:act2</xpath>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2</xpath>"
-            "<in>"
-                "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2/o:l13</xpath>"
-                "</inst-id>"
-            "</in>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:rpc1</xpath>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:cont/o:list1/o:act2</path>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:rpc1</path>"
             "<in>"
                 "<module>ops-ref</module>"
                 "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:rpc1/o:l2</xpath>"
+                    "<path xmlns:o=\"urn:ops\">/o:rpc1/o:l2</path>"
                     "<default-module>ops-ref</default-module>"
                 "</inst-id>"
             "</in>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:rpc2</xpath>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:rpc2</path>"
             "<out>"
                 "<module>ops-ref</module>"
             "</out>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:rpc3</xpath>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:notif3</xpath>"
-            "<in>"
+        "</rpc>"
+        "<rpc>"
+            "<path xmlns:o=\"urn:ops\">/o:rpc3</path>"
+        "</rpc>"
+        "<notification>"
+            "<path xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2</path>"
+            "<deps>"
+                "<inst-id>"
+                    "<path xmlns:o=\"urn:ops\">/o:cont/o:cont3/o:notif2/o:l13</path>"
+                "</inst-id>"
+            "</deps>"
+        "</notification>"
+        "<notification>"
+            "<path xmlns:o=\"urn:ops\">/o:notif3</path>"
+            "<deps>"
                 "<module>ops-ref</module>"
                 "<inst-id>"
-                    "<xpath xmlns:o=\"urn:ops\">/o:notif3/o:list2/o:l15</xpath>"
+                    "<path xmlns:o=\"urn:ops\">/o:notif3/o:list2/o:l15</path>"
                     "<default-module>ops</default-module>"
                 "</inst-id>"
-            "</in>"
-        "</op-deps>"
-        "<op-deps>"
-            "<xpath xmlns:o=\"urn:ops\">/o:notif4</xpath>"
-        "</op-deps>"
+            "</deps>"
+        "</notification>"
+        "<notification>"
+            "<path xmlns:o=\"urn:ops\">/o:notif4</path>"
+        "</notification>"
     "</module>"
     );
 
@@ -434,19 +434,19 @@ test_inv_deps(void **state)
         "<name>ietf-routing</name>"
         "<revision>2015-04-17</revision>"
         "<removed/>"
-        "<data-deps>"
+        "<deps>"
             "<module>ietf-interfaces</module>"
-        "</data-deps>"
-        "<op-deps>"
-            "<xpath xmlns:rt=\"urn:ietf:params:xml:ns:yang:ietf-routing\">/rt:fib-route</xpath>"
+        "</deps>"
+        "<inverse-deps>ietf-interfaces</inverse-deps>"
+        "<rpc>"
+            "<path xmlns:rt=\"urn:ietf:params:xml:ns:yang:ietf-routing\">/rt:fib-route</path>"
             "<in>"
                 "<module>ietf-routing</module>"
             "</in>"
             "<out>"
                 "<module>ietf-routing</module>"
             "</out>"
-        "</op-deps>"
-        "<inverse-data-deps>ietf-interfaces</inverse-data-deps>"
+        "</rpc>"
     "</module>"
     );
 
@@ -455,10 +455,10 @@ test_inv_deps(void **state)
         "<name>ietf-interfaces</name>"
         "<revision>2014-05-08</revision>"
         "<removed/>"
-        "<data-deps>"
+        "<deps>"
             "<module>ietf-routing</module>"
-        "</data-deps>"
-        "<inverse-data-deps>ietf-routing</inverse-data-deps>"
+        "</deps>"
+        "<inverse-deps>ietf-routing</inverse-deps>"
     "</module>"
     );
 }
@@ -662,9 +662,9 @@ test_update_module(void **state)
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>rev</name>"
         "<revision>1970-01-01</revision>"
-        "<op-deps>"
-            "<xpath xmlns:r=\"urn:rev\">/r:notif</xpath>"
-        "</op-deps>"
+        "<notification>"
+            "<path xmlns:r=\"urn:rev\">/r:notif</path>"
+        "</notification>"
     "</module>"
     );
 
@@ -711,13 +711,13 @@ test_change_feature(void **state)
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>features</name>"
         "<enabled-feature>feat1</enabled-feature>"
-        "<data-deps><module>test</module></data-deps>"
+        "<deps><module>test</module></deps>"
     "</module>"
     );
     cmp_int_data(st->conn, "test",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>test</name>"
-        "<inverse-data-deps>features</inverse-data-deps>"
+        "<inverse-deps>features</inverse-deps>"
     "</module>"
     );
 
@@ -742,7 +742,7 @@ test_change_feature(void **state)
         "<enabled-feature>feat1</enabled-feature>"
         "<enabled-feature>feat2</enabled-feature>"
         "<enabled-feature>feat3</enabled-feature>"
-        "<data-deps><module>test</module></data-deps>"
+        "<deps><module>test</module></deps>"
     "</module>"
     );
 
@@ -758,7 +758,7 @@ test_change_feature(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(sess, "/features:l3", "val3", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(sess, 0, 0);
+    ret = sr_apply_changes(sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* disable all features */
@@ -808,9 +808,9 @@ test_change_feature(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_delete_item(sess, "/features:l3", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(sess, 0, 0);
+    ret = sr_apply_changes(sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_copy_config(sess, NULL, SR_DS_STARTUP, 0, 0);
+    ret = sr_copy_config(sess, NULL, SR_DS_STARTUP, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_session_stop(sess);
 
@@ -971,16 +971,16 @@ test_foreign_aug(void **state)
     cmp_int_data(st->conn, "aug",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>aug</name>"
-        "<inverse-data-deps>aug-trg</inverse-data-deps>"
+        "<inverse-deps>aug-trg</inverse-deps>"
     "</module>"
     );
 
     cmp_int_data(st->conn, "aug-trg",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>aug-trg</name>"
-        "<data-deps>"
+        "<deps>"
             "<module>aug</module>"
-        "</data-deps>"
+        "</deps>"
     "</module>"
     );
 
@@ -1021,16 +1021,16 @@ test_foreign_aug(void **state)
     cmp_int_data(st->conn, "aug",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>aug</name>"
-        "<inverse-data-deps>aug-trg</inverse-data-deps>"
+        "<inverse-deps>aug-trg</inverse-deps>"
     "</module>"
     );
 
     cmp_int_data(st->conn, "aug-trg",
     "<module xmlns=\"http://www.sysrepo.org/yang/sysrepo\">"
         "<name>aug-trg</name>"
-        "<data-deps>"
+        "<deps>"
             "<module>aug</module>"
-        "</data-deps>"
+        "</deps>"
     "</module>"
     );
 
