@@ -35,6 +35,7 @@
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
+#include "common.h"
 #include "tests/config.h"
 #include "sysrepo.h"
 
@@ -43,7 +44,7 @@ const time_t start_ts = 1550233816;
 struct state {
     sr_conn_ctx_t *conn;
     sr_session_ctx_t *sess;
-    volatile int cb_called;
+    ATOMIC_T cb_called;
     pthread_barrier_t barrier;
 };
 
@@ -139,7 +140,7 @@ clear_ops(void **state)
     sr_delete_item(st->sess, "/ops-ref:l1", 0);
     sr_delete_item(st->sess, "/ops-ref:l2", 0);
     sr_delete_item(st->sess, "/ops:cont", 0);
-    sr_apply_changes(st->sess, 0, 0);
+    sr_apply_changes(st->sess, 0, 1);
 
     return 0;
 }
@@ -272,7 +273,73 @@ create_ops_notif(void **state)
     return 0;
 }
 
-/* TEST 1 */
+/* TEST */
+static void
+notif_dummy_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const char *xpath, const sr_val_t *values,
+        const size_t values_cnt, time_t timestamp, void *private_data)
+{
+    (void)session;
+    (void)notif_type;
+    (void)xpath;
+    (void)values;
+    (void)values_cnt;
+    (void)timestamp;
+    (void)private_data;
+}
+
+static void
+test_input_parameters(void **state)
+{
+    struct state *st = (struct state *)*state;
+    sr_subscription_ctx_t *subscr;
+    struct lyd_node *input;
+    int ret;
+
+    /* invalid xpath */
+    ret = sr_event_notif_subscribe(st->sess, "ops", "\\ops:notif3", 0, 0, notif_dummy_cb, NULL, 0, &subscr);
+    assert_int_equal(ret, SR_ERR_LY);
+
+    /* non-existing module in xpath */
+    ret = sr_event_notif_subscribe(st->sess, "no-mod", NULL, 0, 0, notif_dummy_cb, NULL, 0,  &subscr);
+    assert_int_equal(ret, SR_ERR_NOT_FOUND);
+
+    /* invalid option(SR_SUBSCR_CTX_REUSE) when subscription NULL */
+    subscr = NULL;
+    ret = sr_event_notif_subscribe(st->sess, "ops", NULL, 0, 0, notif_dummy_cb, NULL, SR_SUBSCR_CTX_REUSE, &subscr);
+    assert_int_equal(ret, SR_ERR_OK);
+
+    /* non-existing notification in module */
+    ret = sr_event_notif_subscribe(st->sess, "test", NULL, 0, 0, notif_dummy_cb, NULL, 0, &subscr);
+    assert_int_equal(ret, SR_ERR_NOT_FOUND);
+
+    /* non-existing notification node in xpath */
+    ret = sr_event_notif_subscribe(st->sess, "ops", "/ops:rpc1", 0, 0, notif_dummy_cb, NULL, 0, &subscr);
+    assert_int_equal(ret, SR_ERR_INVAL_ARG);
+
+    /* data tree must be created with the session connection libyang context */
+    struct ly_ctx *ctx = ly_ctx_new(TESTS_DIR"/files/", 0);
+    assert_non_null(ctx);
+    const struct lys_module *mod = lys_parse_path(ctx, TESTS_DIR"/files/simple.yang", LYS_IN_YANG);
+    assert_non_null(mod);
+    input = lyd_new_path(NULL, ctx, "/simple:ac1", NULL, 0, LYD_PATH_OPT_NOPARENTRET);
+    assert_non_null(input);
+    ret = sr_event_notif_send_tree(st->sess, input);
+    assert_int_equal(ret, SR_ERR_INVAL_ARG);
+    lyd_free_withsiblings(input);
+    ly_ctx_destroy(ctx, NULL);
+
+    /* data tree not a valid notification invovation */
+    input = lyd_new_path(NULL, sr_get_context(st->conn), "/ops:cont/list1[k='key']/cont2", NULL, 0, LYD_PATH_OPT_NOPARENTRET);
+    assert_non_null(input);
+    ret = sr_event_notif_send_tree(st->sess, input);
+    assert_int_equal(ret, SR_ERR_INVAL_ARG);
+    for ( ; input->parent; input = input->parent);
+    lyd_free_withsiblings(input);
+
+    sr_unsubscribe(subscr);
+}
+
+/* TEST */
 static void
 notif_simple_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const char *xpath, const sr_val_t *values,
         const size_t values_cnt, time_t timestamp, void *private_data)
@@ -282,7 +349,7 @@ notif_simple_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type,
     (void)timestamp;
 
     assert_int_equal(notif_type, SR_EV_NOTIF_REALTIME);
-    assert_int_equal(sr_session_get_nc_id(session), 1000);
+    assert_int_equal(sr_session_get_event_nc_id(session), 1000);
 
     /* check input data */
     if (!strcmp(xpath, "/ops:notif3")) {
@@ -299,7 +366,7 @@ notif_simple_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type,
     }
 
     /* signal that we were called */
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     pthread_barrier_wait(&st->barrier);
 }
 
@@ -326,7 +393,7 @@ test_simple(void **state)
     sr_val_t input[2];
     int ret;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* subscribe */
     ret = sr_event_notif_subscribe(st->sess, "ops", NULL, 0, 0, notif_simple_cb, st, 0, &subscr);
@@ -339,7 +406,7 @@ test_simple(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /*
@@ -390,7 +457,7 @@ test_simple(void **state)
 
     /* wait for the callback */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     /*
      * create the second notification
@@ -420,14 +487,14 @@ test_simple(void **state)
 
     /* wait for the callback */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 2);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
 
     sr_unsubscribe(subscr);
     sr_unsubscribe(subscr2);
     sr_unsubscribe(subscr3);
 }
 
-/* TEST 2 */
+/* TEST */
 static void
 notif_stop_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const struct lyd_node *notif,
         time_t timestamp, void *private_data)
@@ -437,7 +504,7 @@ notif_stop_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, co
     (void)session;
     (void)timestamp;
 
-    switch (st->cb_called) {
+    switch (ATOMIC_LOAD_RELAXED(st->cb_called)) {
     case 0:
         assert_int_equal(notif_type, SR_EV_NOTIF_STOP);
         assert_null(notif);
@@ -447,7 +514,7 @@ notif_stop_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, co
     }
 
     /* signal that we were called */
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     pthread_barrier_wait(&st->barrier);
 }
 
@@ -458,7 +525,7 @@ test_stop(void **state)
     sr_subscription_ctx_t *subscr;
     int ret;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* subscribe and expect only the stop notification */
     ret = sr_event_notif_subscribe_tree(st->sess, "ops", NULL, time(NULL) - 2, time(NULL) - 1, notif_stop_cb, st, 0, &subscr);
@@ -466,12 +533,12 @@ test_stop(void **state)
 
     /* wait for the stop notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     sr_unsubscribe(subscr);
 }
 
-/* TEST 3 */
+/* TEST */
 static void
 notif_replay_simple_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const struct lyd_node *notif,
         time_t timestamp, void *private_data)
@@ -481,7 +548,7 @@ notif_replay_simple_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif
     (void)session;
     (void)timestamp;
 
-    switch (st->cb_called) {
+    switch (ATOMIC_LOAD_RELAXED(st->cb_called)) {
     case 0:
         assert_int_equal(notif_type, SR_EV_NOTIF_REPLAY);
         assert_non_null(notif);
@@ -500,7 +567,7 @@ notif_replay_simple_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif
     }
 
     /* signal that we were called */
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     pthread_barrier_wait(&st->barrier);
 }
 
@@ -513,12 +580,12 @@ test_replay_simple(void **state)
     time_t cur_ts;
     int ret;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* set some data needed for validation */
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to the data so they are actually present in operational */
@@ -548,12 +615,12 @@ test_replay_simple(void **state)
     pthread_barrier_wait(&st->barrier);
     pthread_barrier_wait(&st->barrier);
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 3);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
 
     sr_unsubscribe(subscr);
 }
 
-/* TEST 4 */
+/* TEST */
 static void
 notif_replay_interval_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const struct lyd_node *notif,
         time_t timestamp, void *private_data)
@@ -563,7 +630,7 @@ notif_replay_interval_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t not
     (void)session;
     (void)timestamp;
 
-    switch (st->cb_called) {
+    switch (ATOMIC_LOAD_RELAXED(st->cb_called)) {
     case 0:
         assert_int_equal(notif_type, SR_EV_NOTIF_REPLAY);
         assert_non_null(notif);
@@ -666,7 +733,7 @@ notif_replay_interval_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t not
     }
 
     /* signal that we were called */
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     pthread_barrier_wait(&st->barrier);
 }
 
@@ -677,7 +744,7 @@ test_replay_interval(void **state)
     sr_subscription_ctx_t *subscr;
     int ret, i = 0;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* subscribe to the first replay interval */
     ret = sr_event_notif_subscribe_tree(st->sess, "ops", NULL, start_ts + 2, start_ts + 13, notif_replay_interval_cb, st,
@@ -688,7 +755,7 @@ test_replay_interval(void **state)
     for (; i < 9; ++i) {
         pthread_barrier_wait(&st->barrier);
     }
-    assert_int_equal(st->cb_called, i);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), i);
 
     /* subscribe to the second replay interval */
     ret = sr_event_notif_subscribe_tree(st->sess, "ops", NULL, start_ts - 20, start_ts + 4, notif_replay_interval_cb, st,
@@ -699,7 +766,7 @@ test_replay_interval(void **state)
     for (; i < 13; ++i) {
         pthread_barrier_wait(&st->barrier);
     }
-    assert_int_equal(st->cb_called, i);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), i);
 
     /* subscribe to the third replay interval */
     ret = sr_event_notif_subscribe_tree(st->sess, "ops", NULL, start_ts + 9, start_ts + 40, notif_replay_interval_cb, st,
@@ -710,12 +777,12 @@ test_replay_interval(void **state)
     for (; i < 20; ++i) {
         pthread_barrier_wait(&st->barrier);
     }
-    assert_int_equal(st->cb_called, i);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), i);
 
     sr_unsubscribe(subscr);
 }
 
-/* TEST 5 */
+/* TEST */
 static void
 notif_no_replay_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const struct lyd_node *notif,
         time_t timestamp, void *private_data)
@@ -725,7 +792,7 @@ notif_no_replay_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_typ
     (void)session;
     (void)timestamp;
 
-    switch (st->cb_called) {
+    switch (ATOMIC_LOAD_RELAXED(st->cb_called)) {
     case 0:
         assert_int_equal(notif_type, SR_EV_NOTIF_REPLAY_COMPLETE);
         assert_null(notif);
@@ -740,7 +807,7 @@ notif_no_replay_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_typ
     }
 
     /* signal that we were called */
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     pthread_barrier_wait(&st->barrier);
 }
 
@@ -752,12 +819,12 @@ test_no_replay(void **state)
     struct lyd_node *notif;
     int ret;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* set some data needed for validation */
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to the data so they are actually present in operational */
@@ -777,7 +844,7 @@ test_no_replay(void **state)
 
     /* wait for the complete notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     /* send the realtime notification */
     ret = sr_event_notif_send_tree(st->sess, notif);
@@ -786,12 +853,12 @@ test_no_replay(void **state)
 
     /* wait for the realtime notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 2);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
 
     sr_unsubscribe(subscr);
 }
 
-/* TEST 6 */
+/* TEST */
 static void
 notif_config_change_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const struct lyd_node *notif,
         time_t timestamp, void *private_data)
@@ -807,7 +874,7 @@ notif_config_change_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif
     (void)session;
     (void)timestamp;
 
-    switch (st->cb_called) {
+    switch (ATOMIC_LOAD_RELAXED(st->cb_called)) {
     case 0:
         lyd_print_mem(&str1, notif->child->next, LYD_XML, LYP_WITHSIBLINGS);
         str2 =
@@ -913,7 +980,7 @@ notif_config_change_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif
     }
 
     /* signal that we were called */
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     pthread_barrier_wait(&st->barrier);
 }
 
@@ -924,7 +991,7 @@ test_notif_config_change(void **state)
     sr_subscription_ctx_t *subscr;
     int ret;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* subscribe to netconf-config-change */
     ret = sr_event_notif_subscribe_tree(st->sess, "ietf-netconf-notifications",
@@ -934,12 +1001,12 @@ test_notif_config_change(void **state)
     /* repeatedly set some data and check the notification */
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* wait for the notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     ret = sr_set_item_str(st->sess, "/test:test-leaf", "52", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
@@ -951,23 +1018,23 @@ test_notif_config_change(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops-ref:l2", "other-val", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* wait for the notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 2);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
 
     ret = sr_move_item(st->sess, "/test:cont/l2[k='one']", SR_MOVE_AFTER, "[k='two:three']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops-ref:l1", "val2", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* wait for the notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 3);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
 
     ret = sr_delete_item(st->sess, "/test:test-leaf", 0);
     assert_int_equal(ret, SR_ERR_OK);
@@ -975,17 +1042,17 @@ test_notif_config_change(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_delete_item(st->sess, "/test:cont/l2[k='two:three']", 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* wait for the notification */
     pthread_barrier_wait(&st->barrier);
-    assert_int_equal(st->cb_called, 4);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 4);
 
     sr_unsubscribe(subscr);
 }
 
-/* TEST 7 */
+/* TEST */
 static void
 test_notif_buffer(void **state)
 {
@@ -1021,54 +1088,95 @@ test_notif_buffer(void **state)
     lyd_free_withsiblings(notif);
 }
 
+/* TEST */
 static void
-test_input_parameters(void **state)
+notif_suspend_cb(sr_session_ctx_t *session, const sr_ev_notif_type_t notif_type, const char *xpath, const sr_val_t *values,
+        const size_t values_cnt, time_t timestamp, void *private_data)
+{
+    struct state *st = (struct state *)private_data;
+
+    (void)values;
+    (void)values_cnt;
+    (void)timestamp;
+
+    switch (ATOMIC_LOAD_RELAXED(st->cb_called)) {
+    case 0:
+        assert_int_equal(sr_session_get_event_nc_id(session), 1000);
+        assert_int_equal(notif_type, SR_EV_NOTIF_REALTIME);
+        assert_string_equal(xpath, "/ops:notif4");
+        break;
+    case 1:
+        assert_int_equal(sr_session_get_event_nc_id(session), 0);
+        assert_int_equal(notif_type, SR_EV_NOTIF_SUSPENDED);
+        assert_null(xpath);
+        break;
+    case 2:
+        assert_int_equal(sr_session_get_event_nc_id(session), 0);
+        assert_int_equal(notif_type, SR_EV_NOTIF_RESUMED);
+        assert_null(xpath);
+        break;
+    case 3:
+        assert_int_equal(sr_session_get_event_nc_id(session), 1000);
+        assert_int_equal(notif_type, SR_EV_NOTIF_REALTIME);
+        assert_string_equal(xpath, "/ops:notif4");
+        break;
+    default:
+        fail();
+    }
+
+    /* signal that we were called */
+    ATOMIC_INC_RELAXED(st->cb_called);
+    if (notif_type == SR_EV_NOTIF_REALTIME) {
+        pthread_barrier_wait(&st->barrier);
+    }
+}
+
+static void
+test_suspend(void **state)
 {
     struct state *st = (struct state *)*state;
     sr_subscription_ctx_t *subscr;
-    struct lyd_node *input;
     int ret;
+    uint32_t sub_id;
 
-    /* invalid xpath */
-    ret = sr_event_notif_subscribe(st->sess, "ops", "\\ops:notif3", 0, 0, notif_simple_cb, st, 0, &subscr);
-    assert_int_equal(ret, SR_ERR_LY);
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
-    /* non-existing module in xpath */
-    ret = sr_event_notif_subscribe(st->sess, "no-mod", NULL, 0, 0, notif_simple_cb, st, 0,  &subscr);
-    assert_int_equal(ret, SR_ERR_NOT_FOUND);
-
-    /* invalid option(SR_SUBSCR_CTX_REUSE) when subscription NULL */
-    subscr = NULL;
-    ret = sr_event_notif_subscribe(st->sess, "ops", NULL, 0, 0, notif_simple_cb, st, SR_SUBSCR_CTX_REUSE, &subscr);
+    /* subscribe */
+    ret = sr_event_notif_subscribe(st->sess, "ops", NULL, 0, 0, notif_suspend_cb, st, 0, &subscr);
     assert_int_equal(ret, SR_ERR_OK);
 
-    /* non-existing notification in module */
-    ret = sr_event_notif_subscribe(st->sess, "test", NULL, 0, 0, notif_simple_cb, st, 0, &subscr);
-    assert_int_equal(ret, SR_ERR_NOT_FOUND);
+    /* send a notif */
+    ret = sr_event_notif_send(st->sess, "/ops:notif4", NULL, 0);
+    assert_int_equal(ret, SR_ERR_OK);
 
-    /* non-existing notification node in xpath */
-    ret = sr_event_notif_subscribe(st->sess, "ops", "/ops:rpc1", 0, 0, notif_simple_cb, st, 0, &subscr);
-    assert_int_equal(ret, SR_ERR_INVAL_ARG);
+    /* wait for the callback */
+    pthread_barrier_wait(&st->barrier);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
-    /* data tree must be created with the session connection libyang context */
-    struct ly_ctx *ctx = ly_ctx_new(TESTS_DIR"/files/", 0);
-    assert_non_null(ctx);
-    const struct lys_module *mod = lys_parse_path(ctx, TESTS_DIR"/files/simple.yang", LYS_IN_YANG);
-    assert_non_null(mod);
-    input = lyd_new_path(NULL, ctx, "/simple:ac1", NULL, 0, LYD_PATH_OPT_NOPARENTRET);
-    assert_non_null(input);
-    ret = sr_event_notif_send_tree(st->sess, input);
-    assert_int_equal(ret, SR_ERR_INVAL_ARG);
-    lyd_free_withsiblings(input);
-    ly_ctx_destroy(ctx, NULL);
+    /* suspend */
+    sub_id = sr_event_notif_sub_id_get_last(subscr);
+    ret = sr_event_notif_sub_suspend(subscr, sub_id);
+    assert_int_equal(ret, SR_ERR_OK);
 
-    /* data tree not a valid notification invovation */
-    input = lyd_new_path(NULL, sr_get_context(st->conn), "/ops:cont/list1[k='key']/cont2", NULL, 0, LYD_PATH_OPT_NOPARENTRET);
-    assert_non_null(input);
-    ret = sr_event_notif_send_tree(st->sess, input);
-    assert_int_equal(ret, SR_ERR_INVAL_ARG);
-    for(; input->parent; input = input->parent);
-    lyd_free_withsiblings(input);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
+
+    /* send a notif, it is not delivered */
+    ret = sr_event_notif_send(st->sess, "/ops:notif4", NULL, 0);
+    assert_int_equal(ret, SR_ERR_OK);
+
+    /* resume */
+    ret = sr_event_notif_sub_resume(subscr, sub_id);
+    assert_int_equal(ret, SR_ERR_OK);
+
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
+
+    /* send a notif */
+    ret = sr_event_notif_send(st->sess, "/ops:notif4", NULL, 0);
+    assert_int_equal(ret, SR_ERR_OK);
+
+    /* wait for the callback */
+    pthread_barrier_wait(&st->barrier);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 4);
 
     sr_unsubscribe(subscr);
 }
@@ -1078,6 +1186,7 @@ int
 main(void)
 {
     const struct CMUnitTest tests[] = {
+        cmocka_unit_test(test_input_parameters),
         cmocka_unit_test_teardown(test_simple, clear_ops),
         cmocka_unit_test_setup(test_stop, clear_ops_notif),
         cmocka_unit_test_setup_teardown(test_replay_simple, clear_ops_notif, clear_ops),
@@ -1085,7 +1194,7 @@ main(void)
         cmocka_unit_test_setup_teardown(test_no_replay, clear_ops_notif, clear_ops),
         cmocka_unit_test_teardown(test_notif_config_change, clear_ops),
         cmocka_unit_test(test_notif_buffer),
-        cmocka_unit_test(test_input_parameters),
+        cmocka_unit_test(test_suspend),
     };
 
     setenv("CMOCKA_TEST_ABORT", "1", 1);
