@@ -33,6 +33,7 @@
 #include <cmocka.h>
 #include <libyang/libyang.h>
 
+#include "common.h"
 #include "tests/config.h"
 #include "sysrepo.h"
 #include "utils/values.h"
@@ -40,7 +41,7 @@
 struct state {
     sr_conn_ctx_t *conn;
     sr_session_ctx_t *sess;
-    volatile int cb_called;
+    ATOMIC_T cb_called;
     pthread_barrier_t barrier;
 };
 
@@ -57,7 +58,7 @@ setup(void **state)
     sr_connection_count(&conn_count);
     assert_int_equal(conn_count, 0);
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     pthread_barrier_init(&st->barrier, NULL, 2);
 
     if (sr_connect(0, &(st->conn)) != SR_ERR_OK) {
@@ -132,7 +133,7 @@ clear_ops(void **state)
     sr_delete_item(st->sess, "/ops-ref:l1", 0);
     sr_delete_item(st->sess, "/ops-ref:l2", 0);
     sr_delete_item(st->sess, "/ops:cont", 0);
-    sr_apply_changes(st->sess, 0, 0);
+    sr_apply_changes(st->sess, 0, 1);
 
     return 0;
 }
@@ -151,7 +152,7 @@ rpc_fail_cb(sr_session_ctx_t *session, const char *xpath, const struct lyd_node
     (void)output;
     (void)private_data;
 
-    assert_int_equal(sr_session_get_nc_id(session), 128);
+    assert_int_equal(sr_session_get_event_nc_id(session), 128);
     assert_string_equal(xpath, "/ops:rpc1");
 
     /* check input data tree */
@@ -308,7 +309,7 @@ test_rpc(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops-ref:l2", "l2-val", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /*
@@ -394,6 +395,19 @@ test_rpc(void **state)
 
     sr_free_values(output, output_count);
 
+    /*
+     * try to send a non-existing RPC, expect an error
+     */
+    ret = sr_rpc_send(st->sess, "/ops:invalid", &input, 1, 0, &output, &output_count);
+    assert_int_equal(ret, SR_ERR_LY);
+    ret = sr_get_error(st->sess, &err_info);
+    assert_int_equal(ret, SR_ERR_OK);
+    assert_int_equal(err_info->err_count, 1);
+    assert_string_equal(err_info->err[0].message, "Schema node not found.");
+    assert_string_equal(err_info->err[0].xpath, "/ops:invalid");
+    assert_null(output);
+    assert_int_equal(output_count, 0);
+
     sr_unsubscribe(subscr);
 }
 
@@ -462,7 +476,7 @@ test_action(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops:cont/l12", "l12-val", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to the data so they are actually present in operational */
@@ -574,7 +588,7 @@ test_action_pred(void **state)
     ret = sr_rpc_subscribe_tree(st->sess, "/ops:cont/list1[k='one' or k='two']/cont2/act1", rpc_action_pred_cb, NULL, 0, 0, &subscr);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_rpc_subscribe_tree(st->sess, "/ops:cont/list1[k='three' or k='four']/cont2/act1", rpc_action_pred_cb, NULL,
-            0, SR_SUBSCR_CTX_REUSE, &subscr);
+            1, SR_SUBSCR_CTX_REUSE, &subscr);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* set some data needed for validation and executing the actions */
@@ -588,7 +602,7 @@ test_action_pred(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to the data so they are actually present in operational */
@@ -666,7 +680,7 @@ rpc_multi_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_no
     (void)request_id;
     (void)output;
 
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
     return SR_ERR_OK;
 }
 
@@ -682,10 +696,10 @@ test_multi(void **state)
     ret = sr_rpc_subscribe_tree(st->sess, "/ops:cont/list1/cont2/act1", rpc_multi_cb, st, 0, 0, &subscr);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_rpc_subscribe_tree(st->sess, "/ops:cont/list1[k='one' or k='two']/cont2/act1", rpc_multi_cb, st,
-            0, SR_SUBSCR_CTX_REUSE, &subscr);
+            1, SR_SUBSCR_CTX_REUSE, &subscr);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_rpc_subscribe_tree(st->sess, "/ops:cont/list1[k='two' or k='three' or k='four']/cont2/act1",
-            rpc_multi_cb, st, 0, SR_SUBSCR_CTX_REUSE, &subscr);
+            rpc_multi_cb, st, 2, SR_SUBSCR_CTX_REUSE, &subscr);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* set some data needed for validation and executing the actions */
@@ -699,7 +713,7 @@ test_multi(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to the data so they are actually present in operational */
@@ -717,7 +731,7 @@ test_multi(void **state)
     assert_non_null(node);
 
     /* send action */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     for (; input_op->parent; input_op = input_op->parent);
     lyd_free_withsiblings(input_op);
@@ -725,7 +739,7 @@ test_multi(void **state)
     lyd_free_withsiblings(output_op);
 
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     /*
      * create second action
@@ -738,7 +752,7 @@ test_multi(void **state)
     assert_non_null(node);
 
     /* send action */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     for (; input_op->parent; input_op = input_op->parent);
     lyd_free_withsiblings(input_op);
@@ -746,7 +760,7 @@ test_multi(void **state)
     lyd_free_withsiblings(output_op);
 
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(st->cb_called, 2);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
 
     /*
      * create third action
@@ -759,7 +773,7 @@ test_multi(void **state)
     assert_non_null(node);
 
     /* send action */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     for (; input_op->parent; input_op = input_op->parent);
     lyd_free_withsiblings(input_op);
@@ -767,7 +781,7 @@ test_multi(void **state)
     lyd_free_withsiblings(output_op);
 
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(st->cb_called, 3);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
 
     sr_unsubscribe(subscr);
 }
@@ -787,7 +801,7 @@ rpc_multi_fail0_cb(sr_session_ctx_t *session, const char *op_path, const struct
     (void)input;
     (void)request_id;
 
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
 
     /* create output data in all cases, it should always be freed */
     node = lyd_new_path(output, NULL, "l5", "0", 0, LYD_PATH_OPT_OUTPUT);
@@ -796,14 +810,14 @@ rpc_multi_fail0_cb(sr_session_ctx_t *session, const char *op_path, const struct
     switch (call_no) {
     case 1:
         assert_int_equal(event, SR_EV_RPC);
-        assert_int_equal(st->cb_called, 3);
+        assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
         /* callback fails */
         ret = SR_ERR_NOMEM;
         ++call_no;
         break;
     case 2:
         assert_int_equal(event, SR_EV_RPC);
-        assert_int_equal(st->cb_called, 3);
+        assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
         ++call_no;
         break;
     default:
@@ -827,7 +841,7 @@ rpc_multi_fail1_cb(sr_session_ctx_t *session, const char *op_path, const struct
     (void)input;
     (void)request_id;
 
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
 
     /* create output data in all cases, it should always be freed */
     node = lyd_new_path(output, NULL, "l5", "1", 0, LYD_PATH_OPT_OUTPUT);
@@ -836,25 +850,23 @@ rpc_multi_fail1_cb(sr_session_ctx_t *session, const char *op_path, const struct
     switch (call_no) {
     case 1:
         if (event == SR_EV_RPC) {
-            assert_int_equal(st->cb_called, 2);
+            assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
         } else {
             assert_int_equal(event, SR_EV_ABORT);
-            assert_int_equal(st->cb_called, 5);
+            assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 5);
             ++call_no;
-            /* last callback */
-            pthread_barrier_wait(&st->barrier);
         }
         break;
     case 2:
         assert_int_equal(event, SR_EV_RPC);
-        assert_int_equal(st->cb_called, 2);
+        assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
         /* callback fails */
         ret = SR_ERR_NOT_FOUND;
         ++call_no;
         break;
     case 3:
         assert_int_equal(event, SR_EV_RPC);
-        assert_int_equal(st->cb_called, 2);
+        assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
         ++call_no;
         break;
     default:
@@ -878,7 +890,7 @@ rpc_multi_fail2_cb(sr_session_ctx_t *session, const char *op_path, const struct
     (void)input;
     (void)request_id;
 
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
 
     /* create output data in all cases, it should always be freed */
     node = lyd_new_path(output, NULL, "l5", "2", 0, LYD_PATH_OPT_OUTPUT);
@@ -887,34 +899,32 @@ rpc_multi_fail2_cb(sr_session_ctx_t *session, const char *op_path, const struct
     switch (call_no) {
     case 1:
         if (event == SR_EV_RPC) {
-            assert_int_equal(st->cb_called, 1);
+            assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
         } else {
             assert_int_equal(event, SR_EV_ABORT);
-            assert_int_equal(st->cb_called, 4);
+            assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 4);
             ++call_no;
         }
         break;
     case 2:
         if (event == SR_EV_RPC) {
-            assert_int_equal(st->cb_called, 1);
+            assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
         } else {
             assert_int_equal(event, SR_EV_ABORT);
-            assert_int_equal(st->cb_called, 3);
+            assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
             ++call_no;
-            /* last callback */
-            pthread_barrier_wait(&st->barrier);
         }
         break;
     case 3:
         assert_int_equal(event, SR_EV_RPC);
-        assert_int_equal(st->cb_called, 1);
+        assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
         /* callback fails, last callback (but there is no callback for abort, synchronizing would block) */
         ret = SR_ERR_LOCKED;
         ++call_no;
         break;
     case 4:
         assert_int_equal(event, SR_EV_RPC);
-        assert_int_equal(st->cb_called, 1);
+        assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
         ++call_no;
         break;
     default:
@@ -949,16 +959,14 @@ test_multi_fail(void **state)
     assert_non_null(node);
 
     /* send RPC */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     lyd_free_withsiblings(input_op);
     lyd_free_withsiblings(output_op);
 
-    pthread_barrier_wait(&st->barrier);
-
     /* it should fail with 5 total callback calls */
     assert_int_equal(ret, SR_ERR_CALLBACK_FAILED);
-    assert_int_equal(st->cb_called, 5);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 5);
 
     /*
      * create second RPC
@@ -969,16 +977,14 @@ test_multi_fail(void **state)
     assert_non_null(node);
 
     /* send RPC */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     lyd_free_withsiblings(input_op);
     lyd_free_withsiblings(output_op);
 
-    pthread_barrier_wait(&st->barrier);
-
     /* it should fail with 3 total callback calls */
     assert_int_equal(ret, SR_ERR_CALLBACK_FAILED);
-    assert_int_equal(st->cb_called, 3);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
 
     /*
      * create third RPC
@@ -989,14 +995,14 @@ test_multi_fail(void **state)
     assert_non_null(node);
 
     /* send RPC */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     lyd_free_withsiblings(input_op);
     lyd_free_withsiblings(output_op);
 
     /* it should fail with 1 total callback calls */
     assert_int_equal(ret, SR_ERR_CALLBACK_FAILED);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     /*
      * create fourth RPC
@@ -1007,13 +1013,13 @@ test_multi_fail(void **state)
     assert_non_null(node);
 
     /* send RPC */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     lyd_free_withsiblings(input_op);
 
     /* it should not fail */
     assert_int_equal(ret, SR_ERR_OK);
-    assert_int_equal(st->cb_called, 3);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 3);
 
     /* check output */
     assert_string_equal(output_op->child->schema->name, "l5");
@@ -1023,76 +1029,6 @@ test_multi_fail(void **state)
     sr_unsubscribe(subscr);
 }
 
-/* TEST */
-static int
-rpc_unlocked_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_node *input, sr_event_t event,
-        uint32_t request_id, struct lyd_node *output, void *private_data)
-{
-    struct state *st = (struct state *)private_data;
-    struct lyd_node *node;
-    sr_subscription_ctx_t *tmp;
-    int ret;
-    static int call_no = 1;
-
-    (void)session;
-    (void)op_path;
-    (void)input;
-    (void)request_id;
-
-    ++st->cb_called;
-
-    /* create some output data */
-    node = lyd_new_path(output, NULL, "l5", "0", 0, LYD_PATH_OPT_OUTPUT);
-    assert_non_null(node);
-
-    switch (call_no) {
-    case 1:
-        assert_int_equal(event, SR_EV_RPC);
-        ++call_no;
-        /* subscribe to something and then unsubscribe */
-        ret = sr_rpc_subscribe_tree(st->sess, "/ops:cont/list1/cont2/act1", rpc_unlocked_cb, NULL, 0, 0, &tmp);
-        assert_int_equal(ret, SR_ERR_OK);
-        sr_unsubscribe(tmp);
-        break;
-    default:
-        fail();
-    }
-
-    return SR_ERR_OK;
-}
-
-static void
-test_unlocked(void **state)
-{
-    struct state *st = (struct state *)*state;
-    sr_subscription_ctx_t *subscr;
-    struct lyd_node *node, *input_op, *output_op;
-    int ret;
-
-    /* subscribe with unlocked flag */
-    ret = sr_rpc_subscribe_tree(st->sess, "/ops:rpc3", rpc_unlocked_cb, st, 0, SR_SUBSCR_UNLOCKED, &subscr);
-    assert_int_equal(ret, SR_ERR_OK);
-
-    /*
-     * create first RPC
-     */
-    input_op = lyd_new_path(NULL, sr_get_context(st->conn), "/ops:rpc3", NULL, 0, LYD_PATH_OPT_NOPARENTRET);
-    assert_non_null(input_op);
-    node = lyd_new_path(input_op, NULL, "l4", "val", 0, 0);
-    assert_non_null(node);
-
-    /* send RPC */
-    st->cb_called = 0;
-    ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
-    lyd_free_withsiblings(input_op);
-    lyd_free_withsiblings(output_op);
-
-    assert_int_equal(st->cb_called, 1);
-    assert_int_equal(ret, SR_ERR_OK);
-
-    sr_unsubscribe(subscr);
-}
-
 /* TEST */
 static int
 action_deps_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_node *input, sr_event_t event,
@@ -1107,7 +1043,7 @@ action_deps_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_
     (void)request_id;
     (void)output;
 
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
 
     return SR_ERR_OK;
 }
@@ -1132,10 +1068,10 @@ test_action_deps(void **state)
     assert_non_null(input_op);
 
     /* send action, its parent does not exist so it should fail */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     assert_null(output_op);
-    assert_int_equal(st->cb_called, 0);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 0);
     assert_int_equal(ret, SR_ERR_VALIDATION_FAILED);
 
     /* create the necessary data in operational */
@@ -1144,21 +1080,21 @@ test_action_deps(void **state)
 
     ret = sr_set_item_str(st->sess, "/act:advanced/condition", "true", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     ret = sr_session_switch_ds(st->sess, SR_DS_RUNNING);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* send the action again, should succeed now */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     assert_int_equal(ret, SR_ERR_OK);
     while (output_op->parent) {
         output_op = output_op->parent;
     }
     lyd_free_withsiblings(output_op);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     while (input_op->parent) {
         input_op = input_op->parent;
@@ -1171,14 +1107,14 @@ test_action_deps(void **state)
     assert_non_null(input_op);
 
     /* send the action, should succeed */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     assert_int_equal(ret, SR_ERR_OK);
     while (output_op->parent) {
         output_op = output_op->parent;
     }
     lyd_free_withsiblings(output_op);
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     while (input_op->parent) {
         input_op = input_op->parent;
@@ -1208,10 +1144,10 @@ action_change_config_cb(sr_session_ctx_t *session, const char *xpath, const sr_v
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(session, "/ops:cont/list1[k='val2']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(session, 0, 0);
+    ret = sr_apply_changes(session, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
-    ++st->cb_called;
+    ATOMIC_INC_RELAXED(st->cb_called);
 
     /* create some output data */
     sr_new_values(1, output);
@@ -1239,7 +1175,7 @@ test_action_change_config(void **state)
     /* create the data in running and subscribe to them */
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='val']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_module_change_subscribe(st->sess, "ops", NULL, module_change_dummy_cb, NULL, 0, 0, &subscr2);
     assert_int_equal(ret, SR_ERR_OK);
@@ -1250,14 +1186,14 @@ test_action_change_config(void **state)
     assert_non_null(input_op);
 
     /* send the action */
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
     ret = sr_rpc_send_tree(st->sess, input_op, 0, &output_op);
     while (output_op->parent) {
         output_op = output_op->parent;
     }
     lyd_free_withsiblings(output_op);
 
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     while (input_op->parent) {
@@ -1295,8 +1231,8 @@ rpc_shelve_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_n
     (void)request_id;
 
     /* callback called */
-    ++st->cb_called;
-    if (st->cb_called == 1) {
+    ATOMIC_INC_RELAXED(st->cb_called);
+    if (ATOMIC_LOAD_RELAXED(st->cb_called) == 1) {
         return SR_ERR_CALLBACK_SHELVE;
     }
 
@@ -1314,7 +1250,7 @@ send_rpc_shelve_thread(void *arg)
     struct lyd_node *input_op, *output_op;
     int ret;
 
-    st->cb_called = 0;
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
 
     /* create the RPC */
     input_op = lyd_new_path(NULL, sr_get_context(st->conn), "/ops:rpc3/l4", "vall", 0, 0);
@@ -1363,22 +1299,22 @@ subscribe_rpc_shelve_thread(void *arg)
     pthread_barrier_wait(&st->barrier);
 
     count = 0;
-    while ((st->cb_called < 1) && (count < 1500)) {
+    while ((ATOMIC_LOAD_RELAXED(st->cb_called) < 1) && (count < 1500)) {
         usleep(10000);
         ++count;
     }
-    assert_int_equal(st->cb_called, 1);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
 
     /* callback was shelved, process it again */
     ret = sr_process_events(subscr, NULL, NULL);
     assert_int_equal(ret, SR_ERR_OK);
 
     count = 0;
-    while ((st->cb_called < 2) && (count < 1500)) {
+    while ((ATOMIC_LOAD_RELAXED(st->cb_called) < 2) && (count < 1500)) {
         usleep(10000);
         ++count;
     }
-    assert_int_equal(st->cb_called, 2);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 2);
 
     /* wait for the other thread to finish */
     pthread_barrier_wait(&st->barrier);
@@ -1400,6 +1336,24 @@ test_rpc_shelve(void **state)
     pthread_join(tid[1], NULL);
 }
 
+/* TEST */
+static int
+rpc_dummy_cb(sr_session_ctx_t *session, const char *xpath, const sr_val_t *input, const size_t input_cnt,
+        sr_event_t event, uint32_t request_id, sr_val_t **output, size_t *output_cnt, void *private_data)
+{
+    (void)session;
+    (void)xpath;
+    (void)input;
+    (void)input_cnt;
+    (void)event;
+    (void)request_id;
+    (void)output;
+    (void)output_cnt;
+    (void)private_data;
+
+    return SR_ERR_OK;
+}
+
 static void
 test_input_parameters(void **state)
 {
@@ -1461,6 +1415,13 @@ test_input_parameters(void **state)
     assert_int_equal(ret, SR_ERR_INVAL_ARG);
     for(; input_op->parent; input_op = input_op->parent);
     lyd_free_withsiblings(input_op);
+
+    /* equal priority */
+    ret = sr_rpc_subscribe(st->sess, "/ops:rpc1", rpc_dummy_cb, NULL, 5, 0, &subscr);
+    assert_int_equal(ret, SR_ERR_OK);
+    ret = sr_rpc_subscribe(st->sess, "/ops:rpc1", rpc_dummy_cb, NULL, 5, SR_SUBSCR_CTX_REUSE, &subscr);
+    assert_int_equal(ret, SR_ERR_INVAL_ARG);
+    sr_unsubscribe(subscr);
 }
 
 /* TEST */
@@ -1487,7 +1448,7 @@ test_rpc_action_with_no_thread(void **state)
     assert_int_equal(ret, SR_ERR_OK);
     ret = sr_set_item_str(st->sess, "/ops-ref:l2", "l2-val", NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* create first RPC */
@@ -1527,7 +1488,7 @@ test_rpc_action_with_no_thread(void **state)
     /* set some data needed for validation and executing the actions */
     ret = sr_set_item_str(st->sess, "/ops:cont/list1[k='key']", NULL, NULL, 0);
     assert_int_equal(ret, SR_ERR_OK);
-    ret = sr_apply_changes(st->sess, 0, 0);
+    ret = sr_apply_changes(st->sess, 0, 1);
     assert_int_equal(ret, SR_ERR_OK);
 
     /* subscribe to the data so they are actually present in operational */
@@ -1551,6 +1512,78 @@ test_rpc_action_with_no_thread(void **state)
     sr_unsubscribe(subscr2);
 }
 
+/* TEST */
+static int
+oper_rpc_oper_cb(sr_session_ctx_t *session, const char *module_name, const char *xpath, const char *request_xpath,
+        uint32_t request_id, struct lyd_node **parent, void *private_data)
+{
+    (void)session;
+    (void)module_name;
+    (void)xpath;
+    (void)request_xpath;
+    (void)request_id;
+    (void)parent;
+    (void)private_data;
+
+    fail();
+    return SR_ERR_UNSUPPORTED;
+}
+
+static int
+rpc_rpc_oper_cb(sr_session_ctx_t *session, const char *op_path, const struct lyd_node *input, sr_event_t event,
+        uint32_t request_id, struct lyd_node *output, void *private_data)
+{
+    struct state *st = (struct state *)private_data;
+    struct lyd_node *node;
+
+    (void)session;
+    (void)op_path;
+    (void)input;
+    (void)event;
+    (void)request_id;
+
+    /* callback called */
+    ATOMIC_INC_RELAXED(st->cb_called);
+
+    /* create output data */
+    node = lyd_new_path(output, NULL, "l5", "256", 0, LYD_PATH_OPT_OUTPUT);
+    assert_non_null(node);
+
+    return SR_ERR_OK;
+}
+
+static void
+test_rpc_oper(void **state)
+{
+    struct state *st = (struct state *)*state;
+    sr_subscription_ctx_t *subscr;
+    sr_val_t input, *output;
+    size_t output_count;
+    int ret;
+
+    /* rpc subscribe */
+    ret = sr_rpc_subscribe_tree(st->sess, "/ops:rpc3", rpc_rpc_oper_cb, st, 0, 0, &subscr);
+    assert_int_equal(ret, SR_ERR_OK);
+
+    /* oper subscribe, should not be called */
+    ret = sr_oper_get_items_subscribe(st->sess, "ops", "/ops:cont", oper_rpc_oper_cb, NULL, SR_SUBSCR_CTX_REUSE, &subscr);
+    assert_int_equal(ret, SR_ERR_OK);
+
+    /* create and send the RPC */
+    input.xpath = "/ops:rpc3/l4";
+    input.type = SR_STRING_T;
+    input.data.string_val = "l4-val";
+    input.dflt = 0;
+
+    ATOMIC_STORE_RELAXED(st->cb_called, 0);
+    ret = sr_rpc_send(st->sess, "/ops:rpc3", &input, 1, 0, &output, &output_count);
+    assert_int_equal(ret, SR_ERR_OK);
+    assert_int_equal(ATOMIC_LOAD_RELAXED(st->cb_called), 1);
+
+    sr_free_values(output, output_count);
+    sr_unsubscribe(subscr);
+}
+
 /* MAIN */
 int
 main(void)
@@ -1562,12 +1595,12 @@ main(void)
         cmocka_unit_test_teardown(test_action_pred, clear_ops),
         cmocka_unit_test_teardown(test_multi, clear_ops),
         cmocka_unit_test(test_multi_fail),
-        cmocka_unit_test(test_unlocked),
         cmocka_unit_test(test_action_deps),
         cmocka_unit_test_teardown(test_action_change_config, clear_ops),
         cmocka_unit_test(test_rpc_shelve),
         cmocka_unit_test(test_input_parameters),
         cmocka_unit_test(test_rpc_action_with_no_thread),
+        cmocka_unit_test(test_rpc_oper),
     };
 
     setenv("CMOCKA_TEST_ABORT", "1", 1);
